!FILE       :	airfoildomain.f90
!UNIT NAME  :	main	(PROGRAM)
!DESCRIPTION:	This program reads the UIUC Airfoil Coordinates file 
!				(http://www.ae.uiuc.edu/m-selig/ads/coord_database.html), and
!				creates and input file (*.poly) to be use with Triangle mesher (by Jonathan Shewchuk).
!				The program creates a box around the airfoil.  The output file then can be used
!				staight away with Triangle.				
!WRITTEN BY	:	Joel GUERRERO 
!REVISION	:	REV01-03/2007
!Email		:	joegi.geo@yahoo.com
program main

implicit none



!****************************************************************************************
!LOCAL VARIABLES DEFINITION
!
integer::eofile,count,i
integer::nlines,io
real::max_x,min_x,max_y,min_y,ir
real::min_1,min_2,min_3
real::delta_x,delta_y
real::max_outer_x,min_outer_x,max_outer_y,min_outer_y
real::hole_x,hole_y

real,allocatable,dimension(:,:)::coordinates
real,allocatable,dimension(:,:)::coordinates_scale

real,allocatable,dimension(:)::array_temp

integer,allocatable,dimension(:,:)::edge_connectivity
integer,allocatable,dimension(:,:)::edge_connectivity_scale

real,allocatable,dimension(:,:)::box_coordinates
integer,allocatable,dimension(:,:)::box_edge_connectivity
!****************************************************************************************



!****************************************************************************************
!LOCAL VARIABLES DEFINITION
!Change the name of the file according t the input
!
open(10,file='ah21.dat')
!****************************************************************************************



!*****************************************************************
!FIND END-OF-FILE IN ORDER TO KNOW THE NUMBER OF COORDINATES

count=0

nlines = 0 

do
  read(10,*,iostat=io)
  if (io/=0) exit
  nlines = nlines + 1
end do

!print*, nlines		!JUST FOR TESTING
write(*,*)'The file have',nlines,' continuos lines'

count = nlines

rewind(10)

!*****************************************************************



!*****************************************************************

!Insert manually number of lines in file
!count = 42

!*****************************************************************



!*****************************************************************
!ALLOCATE ARRAYS

allocate(coordinates(2,count-1))
allocate(coordinates_scale(2,count-1))

allocate(array_temp(count-1))

allocate(edge_connectivity(2,count-1))
allocate(edge_connectivity_scale(2,count-1))

allocate(box_coordinates(2,count+3))
allocate(box_edge_connectivity(2,count+3))


coordinates			= 0
array_temp			= 0
edge_connectivity		= 0
box_coordinates			= 0
box_edge_connectivity		= 0

!*****************************************************************



!*****************************************************************
!READ FILE, ASSIGN COORDINATES AND BUILD EDGE CONNECTIVITY LIST (AIRFOIL)

read(10,*)

do i=1,count-1
	read(10,*) coordinates(1,i),coordinates(2,i)
enddo

close(10)


do i=1,count-1
	if(i.ne.count-1) then
		edge_connectivity(1,i)	=	i
		edge_connectivity(2,i)	=	i+1
	else
		edge_connectivity(1,i)	=	i
		edge_connectivity(2,i)	=	i-(count-2)
	endif
enddo

!*****************************************************************



!*****************************************************************
!FIND MIN AND MAX VALUES IN X AND Y

do i=1,count-1
	array_temp(i)=coordinates(1,i)
enddo

max_x	=	maxval(array_temp)
min_x	=	minval(array_temp)


do i=1,count-1
	array_temp(i)	=	coordinates(2,i)
enddo

max_y	=	maxval(array_temp)
min_y	=	minval(array_temp)

!write(*,*)'Maximum value in X ',max_x				!JUST FOR TESTING
!write(*,*)'Minimum value in X ',min_x				!JUST FOR TESTING
!write(*,*)'Maximum value in Y ',max_y				!JUST FOR TESTING
!write(*,*)'Minimum value in Y ',min_y				!JUST FOR TESTING

!Compute delta X and delta Y
delta_x	=	abs(max_x-min_x)
delta_y	=	abs(max_y-min_y)

!*****************************************************************



!*****************************************************************
!ASSIGN HOLE POSITION
!I need to modify this in the future is not foolproof, check with showme is
!the hole is well positioned
!
hole_x	=	min_x+0.1
hole_y	=	min_y+delta_y/2
!*****************************************************************


!*****************************************************************
!BUILD OUTER BOX
!The outer box is 5 times the chord of the airfoil upwind
!The outer box is 10 times the chord of the airfoil downwind
!The outer box is 5 times the chord of the airfoil upwards
!The outer box is 5 times the chord of the airfoil downwards
!Feel free to change this parameters as you like
!
max_outer_x	=	max_x+10*delta_x
min_outer_x	=	min_x-5*delta_x
max_outer_y	=	5*delta_x
min_outer_y	=	-5*delta_x

!write(*,*)max_outer_x						!JUST FOR TESTING
!write(*,*)min_outer_x						!JUST FOR TESTING
!write(*,*)max_outer_y						!JUST FOR TESTING
!write(*,*)min_outer_y						!JUST FOR TESTING

!Assign X coordinates to the box
box_coordinates(1,count)	=	min_outer_x
box_coordinates(1,count+1)	=	max_outer_x
box_coordinates(1,count+2)	=	max_outer_x
box_coordinates(1,count+3)	=	min_outer_x

!Assign X coordinates to the box
box_coordinates(2,count)	=	max_outer_y
box_coordinates(2,count+1)	=	max_outer_y
box_coordinates(2,count+2)	=	min_outer_y
box_coordinates(2,count+3)	=	min_outer_y

!Built edge connectivity list
do i=count,count+3
	if(i.ne.count+3) then
		box_edge_connectivity(1,i)	=	i
		box_edge_connectivity(2,i)	=	i+1
	else
		box_edge_connectivity(1,i)	=	i
		box_edge_connectivity(2,i)	=	count
	endif
enddo

		
!do i=1,4											!JUST FOR TESTING
!	write(*,*)box_coordinates(1,i),box_coordinates(2,i)					!JUST FOR TESTING
!enddo												!JUST FOR TESTING
!do i=1,4											!JUST FOR TESTING
!	write(*,*)box_edge_connectivity(1,i),box_edge_connectivity(1,i)				!JUST FOR TESTING
!enddo												!JUST FOR TESTING

!*****************************************************************



!*****************************************************************
!SCALING AIRFOIL AND EDGE CONNECTIVITY LIST

do i=1, count-1
	coordinates_scale(1,i)=coordinates(1,i)*3.0-0.5
end do
do i=1, count-1
	coordinates_scale(2,i)=coordinates(2,i)*10.5
end do

!do i=1, count-1
!	write(*,*) coordinates_scale(1,i),coordinates_scale(2,i)
!end do
!i+count+3

do i=1,count-1
	if(i.ne.count-1) then
		edge_connectivity_scale(1,i)	=	i+count+3
		edge_connectivity_scale(2,i)	=	i+count+3+1
	else
		edge_connectivity_scale(1,i)	=	i+count+3
		edge_connectivity_scale(2,i)	=	i+count+3-(count-2)
	endif
enddo

!*****************************************************************



!*****************************************************************
!DUMP DATA TO output.poly TO BE USED IN Triangle

open(20,file='output.poly',status='unknown')


!WRITES TOTAL NUMBER OF VERTICES & OTHERS PARAMETERS
write(20,*)'# Number of vertices, 2d domain, attributes and boundary marker'

!write(20,*) 2*count+2,2,0,1	!For refinement zone
write(20,*) count+3,2,0,1

!*****************************************************************



!*****************************************************************
!WRITES AIRFOIL COORDINATES BY DEFAULT THE MARKER IS 2
!Here the markers are as follows: 2=slip BC,3=noslip BC

write(20,*)'# Airfoil coordinates (INNER DOMAIN)'

do i=1,count-1
	write(20,*) i,coordinates(1,i),coordinates(2,i),2
enddo

!*****************************************************************



!*****************************************************************
!WRITES THE OUTER BOX COORDINATES
!Here the markers are as follows: 2=slip BC,3=noslip BC, 4=outlet, 
!5=inlet, 6=inflow profile.

write(20,*)'# Box coordinates (OUTER DOMAIN)'

do i=count,count+3

	if(i.eq.count) then
		write(20,*) i,box_coordinates(1,i),box_coordinates(2,i),5
	endif
	if(i.eq.count+1) then 
		write(20,*) i,box_coordinates(1,i),box_coordinates(2,i),5	!was 4
	endif
	if(i.eq.count+2) then
		write(20,*) i,box_coordinates(1,i),box_coordinates(2,i),5	!was 4
	endif
	if(i.eq.count+3) then
		write(20,*) i,box_coordinates(1,i),box_coordinates(2,i),5
	endif

enddo

!*****************************************************************



!*****************************************************************
!WRITES SCALED AIRFOIL BY DEFAULT THE MARKER IS 0
!
!write(20,*)'# Scaled airfoil coordinates'
!
!do i=1,count-1
!	write(20,*)	i+count+3,coordinates_scale(1,i),coordinates_scale(2,i),0
!enddo
!*****************************************************************



!*****************************************************************
!WRITE TOTAL NUMBER OF EDGES & OTHERS PARAMETERS

write(20,*)'# Number of edges and boundary marker'

!write(20,*) 2*count+2,1		!For refinement zone
write(20,*) count+3,1

!*****************************************************************



!*****************************************************************
!WRITES AIRFOIL EDGE CONNECTIVITY LIST BY DEFAULT THE MARKER IS 2
!Here the markers are as follows: 2=slip BC,3=noslip BC

write(20,*)'# Airfoil edge connectivity list(INNER DOMAIN)'

do i=1,count-1
	write(20,*) i,edge_connectivity(1,i),edge_connectivity(2,i),2
enddo

!*****************************************************************



!*****************************************************************
!WRITES BOX EDGE CONNECTIVITY LIST
!Here the markers are as follows: 2=slip BC,3=noslip BC, 4=outlet, 
!5=inlet, 6=inflow profile.

write(20,*)'# Box edge connectivity list(OUTER DOMAIN)'

do i=count,count+3

	if(i.eq.count) then
		write(20,*) i,box_edge_connectivity(1,i),box_edge_connectivity(2,i),5	!was 4
	endif
	if(i.eq.count+1) then
		write(20,*) i,box_edge_connectivity(1,i),box_edge_connectivity(2,i),4	
	endif
	if(i.eq.count+2) then
		write(20,*) i,box_edge_connectivity(1,i),box_edge_connectivity(2,i),5	!was 4
	endif
	if(i.eq.count+3) then
		write(20,*) i,box_edge_connectivity(1,i),box_edge_connectivity(2,i),5	
	endif

enddo

!*****************************************************************



!*****************************************************************
!WRITES SCALED AIRFOIL EDGE CONNECTIVITY LIST BY DEFAULT THE MARKER IS 0

!write(20,*)'# Scaled Airfoil edge connectivity list'
!
!do i=1,count-1
!	write(20,*) i+count+3,edge_connectivity_scale(1,i),edge_connectivity_scale(2,i),0
!enddo

!*****************************************************************



!*****************************************************************
!WRITES HOLE DEFINITION

write(20,*)'# One hole defining the airfoil'
write(20,*)1
write(20,*)1,hole_x,hole_y

!*****************************************************************



!*****************************************************************
!WRITES REFINEMENT AREA DEFINITION

!min_1=minval(coordinates)
!min_2=minval(coordinates_scale)
!min_3=(min_2-min_1)/2
!write(20,*)'# Refinement area'
!write(20,*)2
!write(20,*)1,min_3,0,0.0001
!write(20,*)1,max_outer_x-0.5,0,0.1

!*****************************************************************



!*****************************************************************
close(20)
!*****************************************************************



end program main


